@@ -8,7 +8,7 @@ from onnx.reference.op_run import OpRun
 
 
 class Optional(OpRun):
-    def _run(self, x=None, type=None):  # type: ignore
+    def _run(self, x=None, type=None):  # type: ignore  # noqa: A002
         if x is not None and type is not None:
             dt = tensor_dtype_to_np_dtype(type)
             if dt != x.dtype:
